@@ -39,6 +39,12 @@ int create_home_dir;            /* Create home directory if it does not
 				   exist */
 int home_dir_mode = S_IRUSR|S_IWUSR|S_IXUSR|S_IRGRP|S_IXGRP|S_IROTH|S_IXOTH;
 
+int mailbox_mode[NS_MAX];
+
+/* Saved command line. */
+int imap4d_argc;                 
+char **imap4d_argv;
+
 enum imap4d_preauth preauth_mode;
 char *preauth_program;
 int preauth_only;
@@ -169,27 +175,16 @@ imap4d_parse_opt (int key, char *arg, struct argp_state *state)
 }
 
 static int
-cb_other (mu_debug_t debug, void *data, char *arg)
-{
-  set_namespace (NS_OTHER, arg);
-  return 0;
-}
-
-static int
-cb_shared (mu_debug_t debug, void *data, char *arg)
-{
-  set_namespace (NS_SHARED, arg);
-  return 0;
-}
-
-static int
-cb_mode (mu_debug_t debug, void *data, char *arg)
+cb_mode (mu_debug_t debug, void *data, mu_config_value_t *val)
 {
   char *p;
-  home_dir_mode = strtoul (arg, &p, 8);
+  if (mu_cfg_assert_value_type (val, MU_CFG_STRING, debug))
+    return 1;
+  home_dir_mode = strtoul (val->v.string, &p, 8);
   if (p[0] || (home_dir_mode & ~0777))
     mu_cfg_format_error (debug, MU_DEBUG_ERROR, 
-                         _("Invalid mode specification: %s"), arg);
+                         _("Invalid mode specification: %s"),
+			 val->v.string);
   return 0;
 }
 
@@ -239,22 +234,24 @@ parse_preauth_scheme (mu_debug_t debug, const char *scheme, mu_url_t url)
    preauth stdio
 */
 static int
-cb_preauth (mu_debug_t debug, void *data, char *arg)
+cb_preauth (mu_debug_t debug, void *data, mu_config_value_t *val)
 {
-  if (strcmp (arg, "stdio") == 0)
+  if (mu_cfg_assert_value_type (val, MU_CFG_STRING, debug))
+    return 1;
+  if (strcmp (val->v.string, "stdio") == 0)
     preauth_mode = preauth_stdio;
-  else if (strcmp (arg, "ident") == 0)
-    return parse_preauth_scheme (debug, arg, NULL);
-  else if (arg[0] == '/')
+  else if (strcmp (val->v.string, "ident") == 0)
+    return parse_preauth_scheme (debug, val->v.string, NULL);
+  else if (val->v.string[0] == '/')
     {
-      preauth_program = xstrdup (arg);
+      preauth_program = xstrdup (val->v.string);
       preauth_mode = preauth_prog;
     }
   else
     {
       mu_url_t url;
       char *scheme;
-      int rc = mu_url_create (&url, arg);
+      int rc = mu_url_create (&url, val->v.string);
 
       if (rc)
 	{
@@ -266,7 +263,7 @@ cb_preauth (mu_debug_t debug, void *data, char *arg)
       if (rc)
 	{
 	  mu_cfg_format_error (debug, MU_DEBUG_ERROR,
-			       "%s: %s", arg, mu_strerror (rc));
+			       "%s: %s", val->v.string, mu_strerror (rc));
 	  return 1;
 	}
 
@@ -286,29 +283,47 @@ cb_preauth (mu_debug_t debug, void *data, char *arg)
     }
   return 0;
 }
-	
+
+static int
+cb_mailbox_mode (mu_debug_t debug, void *data, mu_config_value_t *val)
+{
+  const char *p;
+  if (mu_cfg_assert_value_type (val, MU_CFG_STRING, debug))
+    return 1;
+  if (mu_parse_stream_perm_string ((int *)data, val->v.string, &p))
+    mu_cfg_format_error (debug, MU_DEBUG_ERROR,
+			 _("invalid mode string near %s"), p);
+  return 0;
+}
+
 static struct mu_cfg_param imap4d_cfg_param[] = {
-  { "other-namespace", mu_cfg_callback, NULL, 0, cb_other, 
-    N_("Set other users' namespace.  Argument is a colon-separated list "
-       "of directories comprising the namespace.") },
-  { "shared-namespace", mu_cfg_callback, NULL, 0, cb_shared,
-    N_("Set shared namespace.  Argument is a colon-separated list "
-       "of directories comprising the namespace.") },
-  { "login-disabled", mu_cfg_int, &login_disabled, 0, NULL,
+  { "other-namespace", MU_CFG_LIST_OF(mu_cfg_string), &namespace[NS_OTHER],
+    0, NULL, 
+    N_("Set other users' namespace.") },
+  { "shared-namespace", MU_CFG_LIST_OF(mu_cfg_string), &namespace[NS_SHARED],
+    0, NULL,
+    N_("Set shared namespace.") },
+  { "other-mailbox-mode", mu_cfg_callback, &mailbox_mode[NS_OTHER], 0,
+    cb_mailbox_mode,
+    N_("File mode for mailboxes in other namespace.") },
+  { "shared-mailbox-mode", mu_cfg_callback, &mailbox_mode[NS_SHARED], 0,
+    cb_mailbox_mode,
+    N_("File mode for mailboxes in shared namespace.") },
+  { "login-disabled", mu_cfg_bool, &login_disabled, 0, NULL,
     N_("Disable LOGIN command.") },
   { "create-home-dir", mu_cfg_bool, &create_home_dir, 0, NULL,
     N_("If true, create non-existing user home directories.") },
   { "home-dir-mode", mu_cfg_callback, NULL, 0, cb_mode,
     N_("File mode for creating user home directories (octal)."),
     N_("mode") },
-  { "tls-required", mu_cfg_int, &tls_required, 0, NULL,
+  { "tls-required", mu_cfg_bool, &tls_required, 0, NULL,
     N_("Always require STARTTLS before entering authentication phase.") },
   { "preauth", mu_cfg_callback, NULL, 0, cb_preauth,
     N_("Configure PREAUTH mode.  MODE is one of:\n"
        "  prog:///<full-program-name: string>\n"
        "  ident[://:<port: string-or-number>]\n"
        "  stdio"),
-    N_("MODE") },
+    N_("mode") },
   { "preauth-only", mu_cfg_bool, &preauth_only, 0, NULL,
     N_("Use only preauth mode.  If unable to setup it, disconnect "
        "immediately.") },
@@ -316,6 +331,8 @@ static struct mu_cfg_param imap4d_cfg_param[] = {
     N_("Name of DES keyfile for decoding ecrypted ident responses.") },
   { "ident-entrypt-only", mu_cfg_bool, &ident_encrypt_only, 0, NULL,
     N_("Use only encrypted ident responses.") },
+  { "id-fields", MU_CFG_LIST_OF(mu_cfg_string), &imap4d_id_list, 0, NULL,
+    N_("List of fields to return in response to ID command.") },
   { ".server", mu_cfg_section, NULL, 0, NULL,
     N_("Server configuration.") },
   TCP_WRAPPERS_CONFIG
@@ -325,7 +342,7 @@ static struct mu_cfg_param imap4d_cfg_param[] = {
 int
 imap4d_session_setup0 ()
 {
-  homedir = mu_normalize_path (strdup (auth_data->dir), "/");
+  homedir = mu_normalize_path (mu_strdup (auth_data->dir));
   if (imap4d_check_home_dir (homedir, auth_data->uid, auth_data->gid))
     return 1;
   
@@ -333,7 +350,7 @@ imap4d_session_setup0 ()
     setuid (auth_data->uid);
 
   util_chdir (homedir);
-  namespace_init (homedir);
+  namespace_init_session (homedir);
   mu_diag_output (MU_DIAG_INFO,
 		  _("User `%s' logged in (source: %s)"), auth_data->name,
 		  auth_data->source);
@@ -370,6 +387,7 @@ get_client_address (int fd, struct sockaddr_in *pcs)
 static int
 imap4d_mainloop (int fd, FILE *infile, FILE *outfile)
 {
+  imap4d_tokbuf_t tokp;
   char *text;
   int debug_mode = isatty (fd);
   static int sigtab[] = { SIGILL, SIGBUS, SIGFPE, SIGSEGV, SIGSTOP, SIGPIPE,
@@ -400,14 +418,14 @@ imap4d_mainloop (int fd, FILE *infile, FILE *outfile)
   util_out ((state == STATE_AUTH) ? RESP_PREAUTH : RESP_OK, "%s", text);
   util_flush_output ();
 
+  tokp = imap4d_tokbuf_init ();
   while (1)
     {
-      char *cmd = imap4d_readline ();
+      imap4d_readline (tokp);
       /* check for updates */
       imap4d_sync ();
-      util_do_command (cmd);
+      util_do_command (tokp);
       imap4d_sync ();
-      free (cmd);
       util_flush_output ();
     }
 
@@ -462,6 +480,9 @@ main (int argc, char **argv)
   static int sigtab[] = { SIGILL, SIGBUS, SIGFPE, SIGSEGV, SIGSTOP, SIGPIPE,
 			  SIGABRT };
 
+  imap4d_argc = argc;
+  imap4d_argv = argv;
+  
   /* Native Language Support */
   MU_APP_INIT_NLS ();
 
@@ -504,6 +525,8 @@ main (int argc, char **argv)
   if (tls_required)
     imap4d_capability_add (IMAP_CAPA_XTLSREQUIRED);
 #endif
+
+  namespace_init ();
   
   auth_gssapi_init ();
   auth_gsasl_init ();
