@@ -102,7 +102,7 @@
 
 /* GNU Mailutils -- a suite of utilities for electronic mail
    Copyright (C) 1999, 2000, 2001, 2002, 2005,
-   2006, 2007 Free Software Foundation, Inc.
+   2006, 2007, 2008 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -127,10 +127,13 @@
 #include <assert.h>
 #include <sieve.h>
 
-mu_sieve_machine_t sieve_machine;
-int sieve_error_count;
+mu_sieve_machine_t mu_sieve_machine;
+int mu_sieve_error_count;
 
 static void branch_fixup (size_t start, size_t end);
+static int _sieve_default_error_printer (void *, const char *, va_list);
+static int _sieve_default_parse_error (void *, const char *, int,
+				       const char *, va_list);
 
 
 /* Enabling traces.  */
@@ -153,7 +156,7 @@ static void branch_fixup (size_t start, size_t end);
 
 #if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
 typedef union YYSTYPE
-#line 35 "sieve.y"
+#line 38 "sieve.y"
 {
   char *string;
   size_t number;
@@ -176,7 +179,7 @@ typedef union YYSTYPE
   } branch;
 }
 /* Line 187 of yacc.c.  */
-#line 180 "sieve-gram.c"
+#line 183 "sieve-gram.c"
 	YYSTYPE;
 # define mu_sieve_yystype YYSTYPE /* obsolescent; will be withdrawn */
 # define YYSTYPE_IS_DECLARED 1
@@ -189,7 +192,7 @@ typedef union YYSTYPE
 
 
 /* Line 216 of yacc.c.  */
-#line 193 "sieve-gram.c"
+#line 196 "sieve-gram.c"
 
 #ifdef short
 # undef short
@@ -486,11 +489,11 @@ static const mu_sieve_yytype_int8 mu_sieve_yyrhs[] =
 /* YYRLINE[YYN] -- source line where rule number YYN was defined.  */
 static const mu_sieve_yytype_uint16 mu_sieve_yyrline[] =
 {
-       0,    71,    71,    72,    76,    77,    80,    88,    90,    98,
-     104,   115,   133,   136,   139,   145,   155,   163,   171,   177,
-     184,   195,   203,   205,   209,   213,   221,   226,   245,   252,
-     272,   275,   278,   283,   290,   294,   298,   302,   306,   312,
-     317,   320,   326,   331
+       0,    74,    74,    75,    79,    80,    83,    91,    93,   101,
+     107,   118,   136,   139,   142,   148,   158,   166,   174,   180,
+     187,   198,   206,   208,   212,   216,   224,   229,   248,   255,
+     275,   278,   281,   286,   293,   297,   301,   305,   309,   315,
+     320,   323,   329,   334
 };
 #endif
 
@@ -1434,58 +1437,58 @@ mu_sieve_yyreduce:
   switch (mu_sieve_yyn)
     {
         case 3:
-#line 73 "sieve.y"
+#line 76 "sieve.y"
     { /* to placate bison */ }
     break;
 
   case 6:
-#line 81 "sieve.y"
+#line 84 "sieve.y"
     {
 		 mu_sieve_require ((mu_sieve_yyvsp[(2) - (3)].list));
 		 /*  All the items in $2 are registered in memory_pool,
 		     so we don't free them */
 		 mu_list_destroy (&(mu_sieve_yyvsp[(2) - (3)].list));
-		 (mu_sieve_yyval.pc) = sieve_machine->pc;
+		 (mu_sieve_yyval.pc) = mu_sieve_machine->pc;
 	       }
     break;
 
   case 8:
-#line 91 "sieve.y"
+#line 94 "sieve.y"
     {
-		 sieve_machine->prog[(mu_sieve_yyvsp[(2) - (4)].pc)].pc = (mu_sieve_yyvsp[(4) - (4)].branch).begin - (mu_sieve_yyvsp[(2) - (4)].pc) - 1;
+		 mu_sieve_machine->prog[(mu_sieve_yyvsp[(2) - (4)].pc)].pc = (mu_sieve_yyvsp[(4) - (4)].branch).begin - (mu_sieve_yyvsp[(2) - (4)].pc) - 1;
 		 if ((mu_sieve_yyvsp[(4) - (4)].branch).branch)
-		   branch_fixup ((mu_sieve_yyvsp[(4) - (4)].branch).branch, sieve_machine->pc);
+		   branch_fixup ((mu_sieve_yyvsp[(4) - (4)].branch).branch, mu_sieve_machine->pc);
 	       }
     break;
 
   case 9:
-#line 99 "sieve.y"
+#line 102 "sieve.y"
     {
-		 (mu_sieve_yyval.pc) = sieve_machine->pc;
+		 (mu_sieve_yyval.pc) = mu_sieve_machine->pc;
 	       }
     break;
 
   case 10:
-#line 105 "sieve.y"
+#line 108 "sieve.y"
     {
 		 if ((mu_sieve_yyvsp[(1) - (1)].branch).begin)
-		   sieve_machine->prog[(mu_sieve_yyvsp[(1) - (1)].branch).cond].pc =
-		                  sieve_machine->pc - (mu_sieve_yyvsp[(1) - (1)].branch).cond - 1;
+		   mu_sieve_machine->prog[(mu_sieve_yyvsp[(1) - (1)].branch).cond].pc =
+		                  mu_sieve_machine->pc - (mu_sieve_yyvsp[(1) - (1)].branch).cond - 1;
 		 else
 		   {
-		     (mu_sieve_yyval.branch).begin = sieve_machine->pc;
+		     (mu_sieve_yyval.branch).begin = mu_sieve_machine->pc;
 		     (mu_sieve_yyval.branch).branch = 0;
 		   }
 	       }
     break;
 
   case 11:
-#line 116 "sieve.y"
+#line 119 "sieve.y"
     {
 		 if ((mu_sieve_yyvsp[(1) - (3)].branch).begin)
 		   {
-		     sieve_machine->prog[(mu_sieve_yyvsp[(1) - (3)].branch).cond].pc = (mu_sieve_yyvsp[(3) - (3)].pc) - (mu_sieve_yyvsp[(1) - (3)].branch).cond - 1;
-		     sieve_machine->prog[(mu_sieve_yyvsp[(2) - (3)].pc)].pc = (mu_sieve_yyvsp[(1) - (3)].branch).branch;
+		     mu_sieve_machine->prog[(mu_sieve_yyvsp[(1) - (3)].branch).cond].pc = (mu_sieve_yyvsp[(3) - (3)].pc) - (mu_sieve_yyvsp[(1) - (3)].branch).cond - 1;
+		     mu_sieve_machine->prog[(mu_sieve_yyvsp[(2) - (3)].pc)].pc = (mu_sieve_yyvsp[(1) - (3)].branch).branch;
 		     (mu_sieve_yyval.branch).begin = (mu_sieve_yyvsp[(1) - (3)].branch).begin;
 		     (mu_sieve_yyval.branch).branch = (mu_sieve_yyvsp[(2) - (3)].pc);
 		   }
@@ -1498,14 +1501,14 @@ mu_sieve_yyreduce:
     break;
 
   case 12:
-#line 133 "sieve.y"
+#line 136 "sieve.y"
     {
 		 (mu_sieve_yyval.branch).begin = 0;
 	       }
     break;
 
   case 14:
-#line 140 "sieve.y"
+#line 143 "sieve.y"
     {
 		 (mu_sieve_yyval.branch).begin = (mu_sieve_yyvsp[(2) - (4)].pc); 
 		 (mu_sieve_yyval.branch).branch = (mu_sieve_yyvsp[(1) - (4)].pc);
@@ -1514,10 +1517,10 @@ mu_sieve_yyreduce:
     break;
 
   case 15:
-#line 146 "sieve.y"
+#line 149 "sieve.y"
     {
-		 sieve_machine->prog[(mu_sieve_yyvsp[(1) - (5)].branch).cond].pc = (mu_sieve_yyvsp[(3) - (5)].pc) - (mu_sieve_yyvsp[(1) - (5)].branch).cond - 1;
-		 sieve_machine->prog[(mu_sieve_yyvsp[(2) - (5)].pc)].pc = (mu_sieve_yyvsp[(1) - (5)].branch).branch;
+		 mu_sieve_machine->prog[(mu_sieve_yyvsp[(1) - (5)].branch).cond].pc = (mu_sieve_yyvsp[(3) - (5)].pc) - (mu_sieve_yyvsp[(1) - (5)].branch).cond - 1;
+		 mu_sieve_machine->prog[(mu_sieve_yyvsp[(2) - (5)].pc)].pc = (mu_sieve_yyvsp[(1) - (5)].branch).branch;
 		 (mu_sieve_yyval.branch).begin = (mu_sieve_yyvsp[(1) - (5)].branch).begin;
 		 (mu_sieve_yyval.branch).branch = (mu_sieve_yyvsp[(2) - (5)].pc);
 		 (mu_sieve_yyval.branch).cond = (mu_sieve_yyvsp[(4) - (5)].pc);
@@ -1525,117 +1528,117 @@ mu_sieve_yyreduce:
     break;
 
   case 16:
-#line 156 "sieve.y"
+#line 159 "sieve.y"
     {
-		 sieve_code_instr (instr_branch);
-		 (mu_sieve_yyval.pc) = sieve_machine->pc;
-		 sieve_code_number (0);
+		 mu_sv_code_instr (_mu_sv_instr_branch);
+		 (mu_sieve_yyval.pc) = mu_sieve_machine->pc;
+		 mu_sv_code_number (0);
 	       }
     break;
 
   case 17:
-#line 164 "sieve.y"
+#line 167 "sieve.y"
     {
-		 sieve_code_instr (instr_branch);
-		 (mu_sieve_yyval.pc) = sieve_machine->pc;
-		 sieve_code_number (0);
+		 mu_sv_code_instr (_mu_sv_instr_branch);
+		 (mu_sieve_yyval.pc) = mu_sieve_machine->pc;
+		 mu_sv_code_number (0);
 	       }
     break;
 
   case 18:
-#line 172 "sieve.y"
+#line 175 "sieve.y"
     {
 		 (mu_sieve_yyval.pc) = (mu_sieve_yyvsp[(2) - (3)].pc);
 	       }
     break;
 
   case 19:
-#line 178 "sieve.y"
+#line 181 "sieve.y"
     {
-		 (mu_sieve_yyval.pclist).start = (mu_sieve_yyval.pclist).end = sieve_machine->pc;
-		 if (sieve_code_instr (instr_brz)
-		     || sieve_code_number (0))
+		 (mu_sieve_yyval.pclist).start = (mu_sieve_yyval.pclist).end = mu_sieve_machine->pc;
+		 if (mu_sv_code_instr (_mu_sv_instr_brz)
+		     || mu_sv_code_number (0))
 		   YYERROR;
 	       }
     break;
 
   case 20:
-#line 185 "sieve.y"
+#line 188 "sieve.y"
     {
-		 sieve_machine->prog[(mu_sieve_yyvsp[(1) - (3)].pclist).end+1].pc = sieve_machine->pc;
-		 (mu_sieve_yyvsp[(1) - (3)].pclist).end = sieve_machine->pc;
-		 if (sieve_code_instr (instr_brz)
-		     || sieve_code_number (0))
+		 mu_sieve_machine->prog[(mu_sieve_yyvsp[(1) - (3)].pclist).end+1].pc = mu_sieve_machine->pc;
+		 (mu_sieve_yyvsp[(1) - (3)].pclist).end = mu_sieve_machine->pc;
+		 if (mu_sv_code_instr (_mu_sv_instr_brz)
+		     || mu_sv_code_number (0))
 		   YYERROR;
 		 (mu_sieve_yyval.pclist) = (mu_sieve_yyvsp[(1) - (3)].pclist);
 	       }
     break;
 
   case 21:
-#line 196 "sieve.y"
+#line 199 "sieve.y"
     {
-		 sieve_code_instr (instr_brz);
-		 (mu_sieve_yyval.pc) = sieve_machine->pc;
-		 sieve_code_number (0);
+		 mu_sv_code_instr (_mu_sv_instr_brz);
+		 (mu_sieve_yyval.pc) = mu_sieve_machine->pc;
+		 mu_sv_code_number (0);
 	       }
     break;
 
   case 22:
-#line 204 "sieve.y"
+#line 207 "sieve.y"
     { /* to placate bison */ }
     break;
 
   case 23:
-#line 206 "sieve.y"
+#line 209 "sieve.y"
     {
-		 sieve_code_anyof ((mu_sieve_yyvsp[(3) - (4)].pclist).start);
+		 mu_sv_code_anyof ((mu_sieve_yyvsp[(3) - (4)].pclist).start);
 	       }
     break;
 
   case 24:
-#line 210 "sieve.y"
+#line 213 "sieve.y"
     {
-		 sieve_code_allof ((mu_sieve_yyvsp[(3) - (4)].pclist).start);
+		 mu_sv_code_allof ((mu_sieve_yyvsp[(3) - (4)].pclist).start);
 	       }
     break;
 
   case 25:
-#line 214 "sieve.y"
+#line 217 "sieve.y"
     {
-		 if (sieve_code_instr (instr_not))
+		 if (mu_sv_code_instr (_mu_sv_instr_not))
 		   YYERROR;
 	       }
     break;
 
   case 26:
-#line 221 "sieve.y"
+#line 224 "sieve.y"
     {
-		 (mu_sieve_yyval.pc) = sieve_machine->pc;
+		 (mu_sieve_yyval.pc) = mu_sieve_machine->pc;
 	       }
     break;
 
   case 27:
-#line 227 "sieve.y"
+#line 230 "sieve.y"
     {
-		 mu_sieve_register_t *reg = mu_sieve_test_lookup (sieve_machine,
-							    (mu_sieve_yyvsp[(1) - (1)].command).ident);
-		 (mu_sieve_yyval.pc) = sieve_machine->pc;
+		 mu_sieve_register_t *reg = 
+		        mu_sieve_test_lookup (mu_sieve_machine, (mu_sieve_yyvsp[(1) - (1)].command).ident);
+		 (mu_sieve_yyval.pc) = mu_sieve_machine->pc;
 
 		 if (!reg)
-		   sieve_compile_error (sieve_filename, sieve_line_num,
-                                _("unknown test: %s"),
-				(mu_sieve_yyvsp[(1) - (1)].command).ident);
+		   mu_sv_compile_error (&mu_sieve_locus,
+					_("unknown test: %s"),
+					(mu_sieve_yyvsp[(1) - (1)].command).ident);
 		 else if (!reg->required)
-		   sieve_compile_error (sieve_filename, sieve_line_num,
-                                _("test `%s' has not been required"),
-				(mu_sieve_yyvsp[(1) - (1)].command).ident);
-		 else if (sieve_code_test (reg, (mu_sieve_yyvsp[(1) - (1)].command).args))
+		   mu_sv_compile_error (&mu_sieve_locus,
+					_("test `%s' has not been required"),
+					(mu_sieve_yyvsp[(1) - (1)].command).ident);
+		 else if (mu_sv_code_test (reg, (mu_sieve_yyvsp[(1) - (1)].command).args))
 		   YYERROR;
 	       }
     break;
 
   case 28:
-#line 246 "sieve.y"
+#line 249 "sieve.y"
     {
 		 (mu_sieve_yyval.command).ident = (mu_sieve_yyvsp[(1) - (2)].string);
 		 (mu_sieve_yyval.command).args = (mu_sieve_yyvsp[(2) - (2)].list);
@@ -1643,34 +1646,34 @@ mu_sieve_yyreduce:
     break;
 
   case 29:
-#line 253 "sieve.y"
+#line 256 "sieve.y"
     {
-		 mu_sieve_register_t *reg = mu_sieve_action_lookup (sieve_machine,
-							      (mu_sieve_yyvsp[(1) - (1)].command).ident);
+		 mu_sieve_register_t *reg = 
+		        mu_sieve_action_lookup (mu_sieve_machine, (mu_sieve_yyvsp[(1) - (1)].command).ident);
 		 
-		 (mu_sieve_yyval.pc) = sieve_machine->pc;
+		 (mu_sieve_yyval.pc) = mu_sieve_machine->pc;
 		 if (!reg)
-		   sieve_compile_error (sieve_filename, sieve_line_num,
-                                _("unknown action: %s"),
-				(mu_sieve_yyvsp[(1) - (1)].command).ident);
+		   mu_sv_compile_error (&mu_sieve_locus,
+					_("unknown action: %s"),
+					(mu_sieve_yyvsp[(1) - (1)].command).ident);
 		 else if (!reg->required)
-		   sieve_compile_error (sieve_filename, sieve_line_num,
-                                _("action `%s' has not been required"),
-				(mu_sieve_yyvsp[(1) - (1)].command).ident);
-		 else if (sieve_code_action (reg, (mu_sieve_yyvsp[(1) - (1)].command).args))
+		   mu_sv_compile_error (&mu_sieve_locus,
+					_("action `%s' has not been required"),
+					(mu_sieve_yyvsp[(1) - (1)].command).ident);
+		 else if (mu_sv_code_action (reg, (mu_sieve_yyvsp[(1) - (1)].command).args))
 		   YYERROR;
 	       }
     break;
 
   case 30:
-#line 272 "sieve.y"
+#line 275 "sieve.y"
     {
 		 (mu_sieve_yyval.list) = NULL;
 	       }
     break;
 
   case 32:
-#line 279 "sieve.y"
+#line 282 "sieve.y"
     {
 		 mu_list_create (&(mu_sieve_yyval.list));
 		 mu_list_append ((mu_sieve_yyval.list), (mu_sieve_yyvsp[(1) - (1)].value));
@@ -1678,7 +1681,7 @@ mu_sieve_yyreduce:
     break;
 
   case 33:
-#line 284 "sieve.y"
+#line 287 "sieve.y"
     {
 		 mu_list_append ((mu_sieve_yyvsp[(1) - (2)].list), (mu_sieve_yyvsp[(2) - (2)].value));
 		 (mu_sieve_yyval.list) = (mu_sieve_yyvsp[(1) - (2)].list);
@@ -1686,42 +1689,42 @@ mu_sieve_yyreduce:
     break;
 
   case 34:
-#line 291 "sieve.y"
+#line 294 "sieve.y"
     {
 		 (mu_sieve_yyval.value) = mu_sieve_value_create (SVT_STRING_LIST, (mu_sieve_yyvsp[(1) - (1)].list));
 	       }
     break;
 
   case 35:
-#line 295 "sieve.y"
+#line 298 "sieve.y"
     {
 		 (mu_sieve_yyval.value) = mu_sieve_value_create (SVT_STRING, (mu_sieve_yyvsp[(1) - (1)].string));
                }
     break;
 
   case 36:
-#line 299 "sieve.y"
+#line 302 "sieve.y"
     {
 		 (mu_sieve_yyval.value) = mu_sieve_value_create (SVT_STRING, (mu_sieve_yyvsp[(1) - (1)].string));
 	       }
     break;
 
   case 37:
-#line 303 "sieve.y"
+#line 306 "sieve.y"
     {
 		 (mu_sieve_yyval.value) = mu_sieve_value_create (SVT_NUMBER, &(mu_sieve_yyvsp[(1) - (1)].number));
 	       }
     break;
 
   case 38:
-#line 307 "sieve.y"
+#line 310 "sieve.y"
     {
 		 (mu_sieve_yyval.value) = mu_sieve_value_create (SVT_TAG, (mu_sieve_yyvsp[(1) - (1)].string));
 	       }
     break;
 
   case 39:
-#line 313 "sieve.y"
+#line 316 "sieve.y"
     {
 		 mu_list_create (&(mu_sieve_yyval.list));
 		 mu_list_append ((mu_sieve_yyval.list), (mu_sieve_yyvsp[(1) - (1)].string));
@@ -1729,14 +1732,14 @@ mu_sieve_yyreduce:
     break;
 
   case 41:
-#line 321 "sieve.y"
+#line 324 "sieve.y"
     {
 		 (mu_sieve_yyval.list) = (mu_sieve_yyvsp[(2) - (3)].list);
 	       }
     break;
 
   case 42:
-#line 327 "sieve.y"
+#line 330 "sieve.y"
     {
 		 mu_list_create (&(mu_sieve_yyval.list));
 		 mu_list_append ((mu_sieve_yyval.list), (mu_sieve_yyvsp[(1) - (1)].string));
@@ -1744,7 +1747,7 @@ mu_sieve_yyreduce:
     break;
 
   case 43:
-#line 332 "sieve.y"
+#line 335 "sieve.y"
     {
 		 mu_list_append ((mu_sieve_yyvsp[(1) - (3)].list), (mu_sieve_yyvsp[(3) - (3)].string));
 		 (mu_sieve_yyval.list) = (mu_sieve_yyvsp[(1) - (3)].list);
@@ -1753,7 +1756,7 @@ mu_sieve_yyreduce:
 
 
 /* Line 1267 of yacc.c.  */
-#line 1757 "sieve-gram.c"
+#line 1760 "sieve-gram.c"
       default: break;
     }
   YY_SYMBOL_PRINT ("-> $$ =", mu_sieve_yyr1[mu_sieve_yyn], &mu_sieve_yyval, &mu_sieve_yyloc);
@@ -1967,13 +1970,13 @@ mu_sieve_yyreturn:
 }
 
 
-#line 338 "sieve.y"
+#line 341 "sieve.y"
 
 
 int
 mu_sieve_yyerror (char *s)
 {
-  sieve_compile_error (sieve_filename, sieve_line_num, "%s", s);
+  mu_sv_compile_error (&mu_sieve_locus, "%s", s);
   return 0;
 }
 
@@ -1982,6 +1985,7 @@ mu_sieve_machine_init (mu_sieve_machine_t *pmach, void *data)
 {
   int rc;
   mu_sieve_machine_t mach;
+  size_t level;
   
   mach = malloc (sizeof (*mach));
   if (!mach)
@@ -1997,6 +2001,14 @@ mu_sieve_machine_init (mu_sieve_machine_t *pmach, void *data)
   mach->data = data;
   mach->error_printer = _sieve_default_error_printer;
   mach->parse_error_printer = _sieve_default_parse_error;
+
+  level = mu_global_debug_level ("sieve");
+  if (level)
+    {
+      mu_debug_create (&mach->debug, mach);
+      mu_debug_set_level (mach->debug, level);
+    }
+  
   *pmach = mach;
   return 0;
 }
@@ -2081,9 +2093,14 @@ mu_sieve_set_debug (mu_sieve_machine_t mach, mu_sieve_printf_t debug)
 }
 
 void
-mu_sieve_set_debug_level (mu_sieve_machine_t mach, mu_debug_t dbg, int level)
+mu_sieve_set_debug_object (mu_sieve_machine_t mach, mu_debug_t dbg)
 {
   mach->debug = dbg;
+}
+
+void
+mu_sieve_set_debug_level (mu_sieve_machine_t mach, int level)
+{
   mach->debug_level = level;
 }
 
@@ -2206,22 +2223,22 @@ string_comp (const void *item, const void *value)
 void
 mu_sieve_machine_begin (mu_sieve_machine_t mach, const char *file)
 {
-  sieve_machine = mach;
-  sieve_error_count = 0;
-  sieve_code_instr (NULL);
+  mu_sieve_machine = mach;
+  mu_sieve_error_count = 0;
+  mu_sv_code_instr (NULL);
 
   mu_list_create (&mach->source_list);
   mu_list_set_comparator (mach->source_list, string_comp);
   
-  sieve_register_standard_actions (mach);
-  sieve_register_standard_tests (mach);
-  sieve_register_standard_comparators (mach);
+  mu_sv_register_standard_actions (mach);
+  mu_sv_register_standard_tests (mach);
+  mu_sv_register_standard_comparators (mach);
 }
 
 void
 mu_sieve_machine_finish (mu_sieve_machine_t mach)
 {
-  sieve_code_instr (NULL);
+  mu_sv_code_instr (NULL);
 }
 
 int
@@ -2231,12 +2248,37 @@ mu_sieve_compile (mu_sieve_machine_t mach, const char *name)
   
   mu_sieve_machine_begin (mach, name);
 
-  if (sieve_lex_begin (name) == 0)
+  if (mu_sv_lex_begin (name) == 0)
     {
       rc = mu_sieve_yyparse ();
-      if (sieve_error_count)
+      if (mu_sieve_error_count)
 	rc = 1;
-      sieve_lex_finish ();
+      mu_sv_lex_finish ();
+    }
+  else
+    rc = 1;
+  
+  mu_sieve_machine_finish (mach);
+  if (rc)
+    mu_sieve_machine_destroy (&mach);
+  return rc;
+}
+
+int
+mu_sieve_compile_buffer (mu_sieve_machine_t mach,
+			 const char *buf, int bufsize,
+			 const char *fname, int line)
+{
+  int rc;
+  
+  mu_sieve_machine_begin (mach, fname);
+
+  if (mu_sv_lex_begin_string (buf, bufsize, fname, line) == 0)
+    {
+      rc = mu_sieve_yyparse ();
+      if (mu_sieve_error_count)
+	rc = 1;
+      mu_sv_lex_finish ();
     }
   else
     rc = 1;
@@ -2250,17 +2292,39 @@ mu_sieve_compile (mu_sieve_machine_t mach, const char *name)
 static void
 _branch_fixup (size_t start, size_t end)
 {
-  size_t prev = sieve_machine->prog[start].pc;
+  size_t prev = mu_sieve_machine->prog[start].pc;
   if (!prev)
     return;
   branch_fixup (prev, end);
-  sieve_machine->prog[prev].pc = end - prev - 1;
+  mu_sieve_machine->prog[prev].pc = end - prev - 1;
 }
 
-void
+static void
 branch_fixup (size_t start, size_t end)
 {
   _branch_fixup (start, end);
-  sieve_machine->prog[start].pc = end - start - 1;
+  mu_sieve_machine->prog[start].pc = end - start - 1;
 }
 
+static int
+_sieve_default_error_printer (void *unused, const char *fmt, va_list ap)
+{
+  return mu_verror (fmt, ap);
+}
+
+static int
+_sieve_default_parse_error (void *unused, const char *filename, int lineno,
+			    const char *fmt, va_list ap)
+{
+  mu_debug_t debug;
+
+  mu_diag_get_debug (&debug);
+  if (filename)
+    mu_debug_set_locus (debug, filename, lineno);
+  mu_diag_vprintf (MU_DIAG_ERROR, fmt, ap);
+  mu_diag_printf (MU_DIAG_ERROR, "\n");
+  mu_debug_set_locus (debug, NULL, 0);
+  return 0;
+}
+
+
