@@ -1,9 +1,10 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2002, 2005 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2000, 2001, 2002, 2005, 
+   2007, 2008 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
+   the Free Software Foundation; either version 3, or (at your option)
    any later version.
 
    GNU Mailutils is distributed in the hope that it will be useful,
@@ -17,6 +18,7 @@
    MA 02110-1301 USA */
 
 #include <mail.local.h>
+#include "mailutils/libargp.h"
 
 int multiple_delivery;     /* Don't return errors when delivering to multiple
 			      recipients */
@@ -24,7 +26,7 @@ int ex_quota_tempfail;     /* Return temporary failure if mailbox quota is
 			      exceeded. If this variable is not set, mail.local
 			      will return "service unavailable" */
 int exit_code = EX_OK;     /* Exit code to be used */
-uid_t uid;                 /* Current user name */
+uid_t uid;                 /* Current user id */
 char *quotadbname = NULL;  /* Name of mailbox quota database */
 char *quota_query = NULL;  /* SQL query to retrieve mailbox quota */
 
@@ -45,18 +47,15 @@ void close_fds (void);
 int make_tmp (const char *from, mu_mailbox_t *mbx);
 void deliver (mu_mailbox_t msg, char *name);
 void guess_retval (int ec);
-void mailer_err (char *fmt, ...);
 void notify_biff (mu_mailbox_t mbox, char *name, size_t size);
 
 const char *program_version = "mail.local (" PACKAGE_STRING ")";
 static char doc[] =
-/* TRANSLATORS: Please, preserve the vertical tabulation (^K character)
-   in this message */
-N_("GNU mail.local -- the local MDA\
-\v\
-Debug flags are:\n\
+N_("GNU mail.local -- the local MDA")
+"\v"
+N_("Debug flags are:\n\
   g - guimb stack traces\n\
-  T - mailutils traces (MU_DEBUG_TRACE)\n\
+  T - mailutils traces (MU_DEBUG_TRACE0-MU_DEBUG_TRACE9)\n\
   P - network protocols (MU_DEBUG_PROT)\n\
   t - sieve trace (MU_SIEVE_DEBUG_TRACE)\n\
   i - sieve instructions trace (MU_SIEVE_DEBUG_INSTR)\n\
@@ -114,39 +113,90 @@ static struct argp argp = {
 static const char *argp_capa[] = {
   "auth",
   "common",
+  "debug",
   "license",
   "logging",
   "mailbox",
+  "locking",
   "mailer",
-  "sieve",
   NULL
 };
 
 char *from = NULL;
 char *progfile_pattern = NULL;
 char *sieve_pattern = NULL;
+char *saved_envelope;  /* A hack to spare mu_envelope_ calls */
 
 #define D_DEFAULT "9s"
 
+static void
+set_debug_flags (mu_debug_t debug, const char *arg)
+{
+  for (; *arg; arg++)
+    {
+      switch (*arg)
+	{
+	case 'g':
+#ifdef WITH_GUILE
+	  debug_guile = 1;
+#endif
+	  break;
+
+	case 't':
+	  sieve_debug_flags |= MU_SIEVE_DEBUG_TRACE;
+	  break;
+	  
+	case 'i':
+	  sieve_debug_flags |= MU_SIEVE_DEBUG_INSTR;
+	  break;
+	  
+	case 'l':
+	  sieve_enable_log = 1;
+	  break;
+	  
+	case 'T':
+	  debug_flags |= MU_DEBUG_LEVEL_UPTO (MU_DEBUG_TRACE7);
+	  break;
+	  
+	case 'P':
+	  debug_flags |= MU_DEBUG_LEVEL_MASK (MU_DEBUG_PROT);
+	  break;
+
+	default:
+	  if (isdigit (*arg))
+	    debug_level = *arg - '0';
+	  else if (debug)
+	    mu_cfg_format_error (debug, MU_DEBUG_ERROR,
+				 _("%c is not a valid debug flag"),
+				 *arg);
+	  else
+	    mu_error (_("%c is not a valid debug flag"), *arg);
+	  break;
+	}
+    }
+}
+
 static error_t
 parse_opt (int key, char *arg, struct argp_state *state)
 {
+  static struct mu_argp_node_list lst;
+
   switch (key)
     {
     case ARG_MULTIPLE_DELIVERY:
-      multiple_delivery = 1;
+      mu_argp_node_list_new (&lst, "ex-multiple-delivery-success", "yes");
       break;
 
     case ARG_QUOTA_TEMPFAIL:
-      ex_quota_tempfail = 1;
+      mu_argp_node_list_new (&lst, "ex-quota-tempfail", "yes");
       break;
 
     case ARG_MESSAGE_ID_HEADER:
-      message_id_header = arg;
+      mu_argp_node_list_new (&lst, "message-id-header", arg);
       break;
 
     case ARG_QUOTA_QUERY:
-      quota_query = arg;
+      mu_argp_node_list_new (&lst, "quota-query", arg);
       break;
       
     case 'r':
@@ -161,62 +211,31 @@ parse_opt (int key, char *arg, struct argp_state *state)
       
 #ifdef USE_DBM
     case 'q':
-      quotadbname = arg;
+      mu_argp_node_list_new (&lst, "quota-db", arg);
       break;
 #endif
 
 #ifdef WITH_GUILE	
     case 's':
-      progfile_pattern = arg;
+      mu_argp_node_list_new (&lst, "source", arg);
       break;
 #endif
 
     case 'S':
-      sieve_pattern = arg;
+      mu_argp_node_list_new (&lst, "sieve-filter", arg);
+      
       break;
       
     case 'x':
-      do
-	{
-	  if (!arg)
-	    arg = D_DEFAULT;
-	  switch (*arg)
-	    {
-	    case 'g':
-#ifdef WITH_GUILE
-	      debug_guile = 1;
-#endif
-	      break;
-
-	    case 't':
-	      sieve_debug_flags |= MU_SIEVE_DEBUG_TRACE;
-	      break;
-
-	    case 'i':
-	      sieve_debug_flags |= MU_SIEVE_DEBUG_INSTR;
-	      break;
-	      
-	    case 'l':
-	      sieve_enable_log = 1;
-	      break;
-	      
-	    case 'T':
-	      debug_flags |= MU_DEBUG_TRACE;
-	      break;
-
-	    case 'P':
-	      debug_flags |= MU_DEBUG_PROT;
-	      break;
-
-	    default:
-	      if (isdigit (*arg))
-		debug_level = *arg - '0';
-	      else
-		argp_error (state, _("%c is not a valid debug flag"), *arg);
-	      break;
-	    }
-	}
-      while (*++arg);
+      mu_argp_node_list_new (&lst, "debug", arg ? arg : D_DEFAULT);
+      break;
+      
+    case ARGP_KEY_INIT:
+      mu_argp_node_list_init (&lst);
+      break;
+      
+    case ARGP_KEY_FINI:
+      mu_argp_node_list_finish (&lst, NULL, NULL);
       break;
       
     default:
@@ -228,29 +247,78 @@ parse_opt (int key, char *arg, struct argp_state *state)
   return 0;
 }
 
-
+
 static int
-_mu_debug_printer (mu_debug_t unused, size_t level, const char *fmt,
-		   va_list ap)
+cb_debug (mu_debug_t debug, void *data, mu_config_value_t *val)
 {
-  vsyslog ((level == MU_DEBUG_ERROR) ? LOG_ERR : LOG_DEBUG, fmt, ap);
+  if (mu_cfg_assert_value_type (val, MU_CFG_STRING, debug))
+    return 1;
+  set_debug_flags (debug, val->v.string);
   return 0;
 }
 
+struct mu_cfg_param mail_local_cfg_param[] = {
+  { "ex-multiple-delivery-success", mu_cfg_bool, &multiple_delivery, 0, NULL,
+    N_("In case of multiple delivery, exit with code 0 if at least one "
+       "delivery succeeded.") },
+  { "ex-quota-tempfail", mu_cfg_bool, &ex_quota_tempfail, 0, NULL,
+    N_("Indicate temporary failure if the recipient is over his mail quota.")
+  },
+#ifdef USE_DBM
+  { "quota-db", mu_cfg_string, &quotadbname, 0, NULL,
+    N_("Name of DBM quota database file."),
+    N_("file") },
+#endif
+#ifdef USE_SQL
+  { "quota-query", mu_cfg_string, &quota_query, 0, NULL,
+    N_("SQL query to retrieve mailbox quota.  This is deprecated, use "
+       "sql { ... } instead."),
+    N_("query") },
+#endif
+  { "sieve-filter", mu_cfg_string, &sieve_pattern, 0, NULL,
+    N_("File name or name pattern for Sieve filter file."),
+    N_("file-or-pattern") },
+  { "message-id-header", mu_cfg_string, &message_id_header, 0, NULL,
+    N_("When logging Sieve actions, identify messages by the value of "
+       "this header."),
+    N_("name") },
+#ifdef WITH_GUILE
+  { "guile-filter", mu_cfg_string, &progfile_pattern, 0, NULL,
+    N_("File name or name pattern for Guile filter file."),
+    N_("file-or-pattern") },
+#endif
+  { "debug", mu_cfg_callback, NULL, 0, cb_debug,
+    N_("Set mail.local debug level.  Debug level consists of one or more "
+       "of the following letters:\n"
+       "  g - guimb stack traces\n"
+       "  t - sieve trace (MU_SIEVE_DEBUG_TRACE)\n"
+       "  i - sieve instructions trace (MU_SIEVE_DEBUG_INSTR)\n"
+       "  l - sieve action logs\n") },
+  { NULL }
+};
+
+
+
 static int
 _sieve_debug_printer (void *unused, const char *fmt, va_list ap)
 {
-  vsyslog (LOG_DEBUG, fmt, ap);
+  mu_diag_vprintf (MU_DIAG_DEBUG, fmt, ap);
   return 0;
 }
 
 static void
 _sieve_action_log (void *user_name,
-		   const mu_sieve_locus_t *locus, size_t msgno, mu_message_t msg,
+		   const mu_sieve_locus_t *locus, size_t msgno,
+		   mu_message_t msg,
 		   const char *action, const char *fmt, va_list ap)
 {
-  char *text = NULL;
+  int pfx = 0;
+  mu_debug_t debug;
+
+  mu_diag_get_debug (&debug);
+  mu_debug_set_locus (debug, locus->source_file, locus->source_line);
   
+  mu_diag_printf (MU_DIAG_NOTICE, _("(user %s) "), (char*) user_name);
   if (message_id_header)
     {
       mu_header_t hdr = NULL;
@@ -259,51 +327,42 @@ _sieve_action_log (void *user_name,
       if (mu_header_aget_value (hdr, message_id_header, &val) == 0
 	  || mu_header_aget_value (hdr, MU_HEADER_MESSAGE_ID, &val) == 0)
 	{
-	  asprintf (&text, _("%s:%lu: %s on msg %s"),
-		    locus->source_file,
-		    (unsigned long) locus->source_line,
-		    action, val);
+	  pfx = 1;
+	  mu_diag_printf (MU_DIAG_NOTICE, _("%s on msg %s"), action, val);
 	  free (val);
 	}
     }
-  if (text == NULL)
+  
+  if (!pfx)
     {
       size_t uid = 0;
       mu_message_get_uid (msg, &uid);
-      asprintf (&text, _("%s:%lu: %s on msg uid %d"),
-		locus->source_file,
-		(unsigned long)	locus->source_line,
-		action, uid);
+      mu_diag_printf (MU_DIAG_NOTICE, _("%s on msg uid %d"), action, uid);
     }
   
   if (fmt && strlen (fmt))
     {
-      char *diag = NULL;
-      vasprintf (&diag, fmt, ap);
-      syslog (LOG_NOTICE, _("(user %s) %s: %s"), (char*) user_name, text, diag);
-      free (diag);
+      mu_diag_printf (MU_DIAG_NOTICE, "; ");
+      mu_diag_vprintf (MU_DIAG_NOTICE, fmt, ap);
     }
-  else
-    syslog (LOG_NOTICE, _("(user %s) %s"), (char*) user_name, text);
-  free (text);
+  mu_diag_printf (MU_DIAG_NOTICE, "\n");
+  mu_debug_set_locus (debug, NULL, 0);
 }
 
 static int
 _sieve_parse_error (void *user_name, const char *filename, int lineno,
 		    const char *fmt, va_list ap)
 {
-  char *text;
-  vasprintf (&text, fmt, ap);
+  mu_debug_t debug;
+
+  mu_diag_get_debug (&debug);
   if (filename)
-    {
-      char *loc;
-      asprintf (&loc, "%s:%d: ", filename, lineno);
-      syslog (LOG_ERR, "%s: %s", loc, text);
-      free (loc);
-    }
-  else
-    syslog (LOG_ERR, _("(user %s) %s"), (char*)user_name, text);
-  free (text);
+    mu_debug_set_locus (debug, filename, lineno);
+
+  mu_diag_printf (MU_DIAG_NOTICE, _("(user %s) "), (char*) user_name);
+  mu_diag_vprintf (MU_DIAG_NOTICE, fmt, ap);
+  mu_diag_printf (MU_DIAG_NOTICE, "\n");
+  mu_debug_set_locus (debug, NULL, 0);
   return 0;
 }
 
@@ -319,25 +378,33 @@ main (int argc, char *argv[])
   umask (0077);
 
   /* Native Language Support */
-  mu_init_nls ();
+  MU_APP_INIT_NLS ();
 
   /* Default locker settings */
   mu_locker_set_default_flags (MU_LOCKER_PID|MU_LOCKER_RETRY,
-			    mu_locker_assign);
+			       mu_locker_assign);
   mu_locker_set_default_retry_timeout (1);
   mu_locker_set_default_retry_count (300);
 
-  /* Default error code for command line errors */
-  mu_argp_error_code = EX_CONFIG;
   /* Register needed modules */
   MU_AUTH_REGISTER_ALL_MODULES();
-  mu_argp_init (program_version, NULL);
-  mu_sieve_argp_init ();
   /* Parse command line */
-  mu_argp_parse (&argp, &argc, &argv, 0, argp_capa, &arg_index, NULL);
+  mu_gocs_register ("sieve", mu_sieve_module_init);
+  mu_argp_init (program_version, NULL);
+  if (mu_app_init (&argp, argp_capa, mail_local_cfg_param,
+		   argc, argv, 0, &arg_index, NULL))
+    exit (EX_CONFIG);
+
+  uid = getuid ();
+
+  if (uid == 0)
+    {
+      mu_debug_t debug;
+      openlog (MU_LOG_TAG (), LOG_PID, mu_log_facility);
+      mu_diag_get_debug (&debug);
+      mu_debug_set_print (debug, mu_diag_syslog_printer, NULL);
+    }
   
-  openlog ("mail.local", LOG_PID, log_facility);
-  mu_error_set_print (mu_syslog_error_printer);
   if (debug_flags)
     {
       int rc;
@@ -353,28 +420,36 @@ main (int argc, char *argv[])
 		    mu_strerror (rc));
 	  exit (EX_TEMPFAIL);
 	}
-      if ((rc = mu_debug_set_print (mudebug, _mu_debug_printer, NULL)))
-	{
-	  mu_error (_("mu_debug_set_print failed: %s\n"),
-		    mu_strerror (rc));
-	  exit (EX_TEMPFAIL);
-	}
     }
   
-  uid = getuid ();
-
   argc -= arg_index;
   argv += arg_index;
 
   if (!argc)
     {
-      mu_error (_("Missing arguments. Try --help for more info."));
-      return EX_USAGE;
+      if (uid)
+	{
+	  static char *s_argv[2];
+	  struct mu_auth_data *auth = mu_get_auth_by_uid (uid);
+
+	  if (!uid)
+	    {
+	      mu_error (_("Cannot get username"));
+	      return EX_UNAVAILABLE;
+	    }
+	    
+	  s_argv[0] = auth->name;
+	  argv = s_argv;
+	  argc = 1;
+	}
+      else
+	{
+	  mu_error (_("Missing arguments. Try --help for more info."));
+	  return EX_USAGE;
+	}
     }
 
-  /* Register local mbox formats. */
-  mu_registrar_record (mu_mbox_record); 
-  mu_registrar_record (mu_path_record);
+  mu_register_all_formats ();
   /* Possible supported mailers.  */
   mu_registrar_record (mu_sendmail_record);
   mu_registrar_record (mu_smtp_record);
@@ -416,7 +491,7 @@ sieve_test (struct mu_auth_data *auth, mu_mailbox_t mbx)
   if (access (progfile, R_OK))
     {
       if (debug_level > 2)
-	syslog (LOG_DEBUG, _("Access to %s failed: %m"), progfile);
+	mu_diag_output (MU_DIAG_DEBUG, _("Access to %s failed: %m"), progfile);
     }
   else
     {
@@ -430,7 +505,8 @@ sieve_test (struct mu_auth_data *auth, mu_mailbox_t mbx)
       else
 	{
 	  mu_sieve_set_debug (mach, _sieve_debug_printer);
-	  mu_sieve_set_debug_level (mach, mudebug, sieve_debug_flags);
+	  mu_sieve_set_debug_object (mach, mudebug);
+	  mu_sieve_set_debug_level (mach, sieve_debug_flags);
 	  mu_sieve_set_parse_error (mach, _sieve_parse_error);
 	  if (sieve_enable_log)
 	    mu_sieve_set_logger (mach, _sieve_action_log);
@@ -449,7 +525,7 @@ sieve_test (struct mu_auth_data *auth, mu_mailbox_t mbx)
 		  chdir (auth->dir);
 		
 		  rc = mu_sieve_message (mach, msg);
-		  if (rc == 0)
+ 		  if (rc == 0)
 		    rc = mu_attribute_is_deleted (attr) == 0;
 
 		  switch_user_id (auth, 0);
@@ -514,7 +590,8 @@ switch_user_id (struct mu_auth_data *auth, int user)
 # error "No way to reset user privileges?"
 #endif
   if (rc < 0)
-    mailer_err ("setreuid(0, %d): %s (r=%d, e=%d)",
+    mailer_err ("cannot set user priviledes", auth->name,
+		"setreuid(0, %d): %s (r=%d, e=%d)",
 		uid, strerror (errno), getuid (), geteuid ());
   return rc;
 }
@@ -545,13 +622,16 @@ make_tmp (const char *from, mu_mailbox_t *mbox)
   tempfile = mu_tempname (NULL);
   if ((status = mu_file_stream_create (&stream, tempfile, MU_STREAM_RDWR)))
     {
-      mailer_err (_("Unable to open temporary file: %s"), mu_strerror (status));
+      mailer_err ("Temporary failure", NULL,
+		  _("Unable to open temporary file: %s"),
+		  mu_strerror (status));
       exit (exit_code);
     }
 
   if ((status = mu_stream_open (stream)))
     {
-      mailer_err (_("unable to open temporary file: %s"), mu_strerror (status));
+      mailer_err ("Temporary failure", NULL,
+		  _("unable to open temporary file: %s"), mu_strerror (status));
       exit (exit_code);
     }
   
@@ -573,21 +653,27 @@ make_tmp (const char *from, mu_mailbox_t *mbox)
 	      if (from)
 		{
 		  time_t t;
-		  char *ptr;
 		  
 		  time (&t);
-		  asprintf (&ptr, "From %s %s", from, ctime (&t));
-		  status = tmp_write (stream, &offset, ptr, strlen (ptr));
-		  free (ptr);
+		  asprintf (&saved_envelope, "From %s %s", from, ctime (&t));
+		  status = tmp_write (stream, &offset, saved_envelope,
+				      strlen (saved_envelope));
 		}
 	      else
 		{
-		  mailer_err (_("Cannot determine sender address"));
+		  mailer_err ("Cannot determine sender address", NULL,
+			      _("Cannot determine sender address"));
 		  exit (EX_UNAVAILABLE);
 		}
 	      if (auth)
 		mu_auth_data_free (auth);
 	    }
+	  else
+	    {
+	      saved_envelope = strdup (buf);
+	      if (!saved_envelope)
+		status = ENOMEM;
+	    }
 	}
       else if (!memcmp (buf, "From ", 5))
 	{
@@ -600,7 +686,9 @@ make_tmp (const char *from, mu_mailbox_t *mbox)
       
       if (status)
 	{
-	  mailer_err (_("Error writing temporary file: %s"), mu_strerror (status));
+	  mailer_err ("Temporary failure", NULL,
+		      _("Error writing temporary file: %s"),
+		      mu_strerror (status));
 	  mu_stream_destroy (&stream, mu_stream_get_owner (stream));
 	  return status;
 	}
@@ -617,7 +705,8 @@ make_tmp (const char *from, mu_mailbox_t *mbox)
   if (status)
     {
       errno = status;
-      mailer_err (_("Error writing temporary file: %s"), mu_strerror (status));
+      mailer_err ("Error writing temporary file", NULL,
+		  _("Error writing temporary file: %s"), mu_strerror (status));
       mu_stream_destroy (&stream, mu_stream_get_owner (stream));
       return status;
     }
@@ -627,7 +716,9 @@ make_tmp (const char *from, mu_mailbox_t *mbox)
       || (status = mu_mailbox_open (*mbox, MU_STREAM_READ))
       || (status = mu_mailbox_set_stream (*mbox, stream)))
     {
-      mailer_err (_("Error opening temporary file: %s"), mu_strerror (status));
+      mailer_err ("Error opening temporary file", NULL,
+		  _("Error opening temporary file: %s"),
+		  mu_strerror (status));
       mu_stream_destroy (&stream, mu_stream_get_owner (stream));
       return status;
     }
@@ -636,7 +727,8 @@ make_tmp (const char *from, mu_mailbox_t *mbox)
   if (status)
     {
       errno = status;
-      mailer_err (_("Error creating temporary message: %s"),
+      mailer_err ("Error collecting message", NULL,
+		  _("Error creating temporary message: %s"),
 		  mu_strerror (status));
       mu_stream_destroy (&stream, mu_stream_get_owner (stream));
       return status;
@@ -649,6 +741,7 @@ void
 deliver (mu_mailbox_t imbx, char *name)
 {
   mu_mailbox_t mbox;
+  mu_message_t msg;
   char *path;
   mu_url_t url = NULL;
   mu_locker_t lock;
@@ -661,11 +754,14 @@ deliver (mu_mailbox_t imbx, char *name)
   auth = mu_get_auth_by_name (name);
   if (!auth)
     {
-      mailer_err (_("%s: no such user"), name);
+      mailer_err ("no such user", name,
+		  _("%s: no such user"), name);
       exit_code = EX_UNAVAILABLE;
       return;
     }
-
+  if (uid)
+    auth->change_uid = 0;
+  
   if (!sieve_test (auth, imbx))
     {
       exit_code = EX_OK;
@@ -673,9 +769,19 @@ deliver (mu_mailbox_t imbx, char *name)
       return;
     }
 
-  if ((status = mu_mailbox_get_stream (imbx, &istream)) != 0)
+  if ((status = mu_mailbox_get_message (imbx, 1, &msg)) != 0)
     {
-      mailer_err (_("Cannot get input message stream: %s"),
+      mailer_err ("cannot read message", name,
+		  _("Cannot get message from the temporary mailbox: %s"),
+		  mu_strerror (status));
+      mu_auth_data_free (auth);
+      return;
+    }
+
+  if ((status = mu_message_get_stream (msg, &istream)) != 0)
+    {
+      mailer_err ("cannot read message", name,
+		  _("Cannot get input message stream: %s"),
 		  mu_strerror (status));
       mu_auth_data_free (auth);
       return;
@@ -683,7 +789,8 @@ deliver (mu_mailbox_t imbx, char *name)
   
   if ((status = mu_mailbox_create (&mbox, auth->mailbox)) != 0)
     {
-      mailer_err (_("Cannot open mailbox %s: %s"),
+      mailer_err ("cannot read message", name,
+		  _("Cannot open mailbox %s: %s"),
 		  auth->mailbox, mu_strerror (status));
       mu_auth_data_free (auth);
       return;
@@ -702,7 +809,8 @@ deliver (mu_mailbox_t imbx, char *name)
     return;
   if (status != 0)
     {
-      mailer_err (_("Cannot open mailbox %s: %s"), path, mu_strerror (status));
+      mailer_err ("cannot open mailbox", name,
+		  _("Cannot open mailbox %s: %s"), path, mu_strerror (status));
       mu_mailbox_destroy (&mbox);
       return;
     }
@@ -713,7 +821,9 @@ deliver (mu_mailbox_t imbx, char *name)
 
   if (status)
     {
-      mailer_err (_("Cannot lock mailbox `%s': %s"), path, mu_strerror (status));
+      mailer_err ("cannot lock mailbox", name,
+		  _("Cannot lock mailbox `%s': %s"), path,
+		  mu_strerror (status));
       mu_mailbox_destroy (&mbox);
       exit_code = EX_TEMPFAIL;
       return;
@@ -721,7 +831,8 @@ deliver (mu_mailbox_t imbx, char *name)
 
   if ((status = mu_mailbox_get_stream (mbox, &ostream)))
     {
-      mailer_err (_("Cannot get stream for mailbox %s: %s"),
+      mailer_err ("cannot access mailbox", name,
+		  _("Cannot get stream for mailbox %s: %s"),
 		  path, mu_strerror (status));
       mu_mailbox_destroy (&mbox);
       return;
@@ -729,7 +840,8 @@ deliver (mu_mailbox_t imbx, char *name)
 
   if ((status = mu_stream_size (ostream, &size)))
     {
-      mailer_err (_("Cannot get stream size (mailbox %s): %s"),
+      mailer_err ("cannot access mailbox", name,
+		  _("Cannot get stream size (mailbox %s): %s"),
 		  path, mu_strerror (status));
       mu_mailbox_destroy (&mbox);
       return;
@@ -737,13 +849,14 @@ deliver (mu_mailbox_t imbx, char *name)
 
 #if defined(USE_MAILBOX_QUOTAS)
   {
-    size_t n;
+    mu_off_t n;
     mu_off_t isize;
 
-    switch (check_quota (name, size, &n))
+    switch (check_quota (auth, size, &n))
       {
       case MQUOTA_EXCEEDED:
-	mailer_err (_("%s: mailbox quota exceeded for this recipient"), name);
+	mailer_err ("mailbox quota exceeded for this recipient", name,
+		    _("%s: mailbox quota exceeded for this recipient"), name);
 	exit_code = EX_QUOTA();
 	failed++;
 	break;
@@ -754,14 +867,17 @@ deliver (mu_mailbox_t imbx, char *name)
       default:
 	if ((status = mu_stream_size (istream, &isize)))
 	  {
-	    mailer_err (_("Cannot get stream size (input message): %s"),
+	    mailer_err ("cannot access mailbox", name,
+			_("Cannot get stream size (input message %s): %s"),
 			path, mu_strerror (status));
 	    exit_code = EX_UNAVAILABLE;
 	    failed++;
 	  }
 	else if (isize > n)
 	  {
-	    mailer_err (_("%s: message would exceed maximum mailbox size for this recipient"),
+	    mailer_err ("message would exceed maximum mailbox size for this recipient",
+			name,
+			_("%s: message would exceed maximum mailbox size for this recipient"),
 			name);
 	    exit_code = EX_QUOTA();
 	    failed++;
@@ -773,9 +889,7 @@ deliver (mu_mailbox_t imbx, char *name)
   
   if (!failed && switch_user_id (auth, 1) == 0)
     {
-      mu_off_t ioff = 0;
-      mu_off_t off = size;
-      size_t nwr, nrd;
+      size_t nrd;
       char *buf = NULL;
       mu_off_t bufsize = 1024;
 
@@ -788,22 +902,28 @@ deliver (mu_mailbox_t imbx, char *name)
 	  status = errno = ENOMEM;
 	  failed++;
 	}
+      else if ((status = mu_stream_seek (ostream, size, SEEK_SET)) != 0
+	       || (status = mu_stream_seek (istream, 0, SEEK_SET) != 0))
+	/* nothing: the error will be reported later */;
       else
 	{
-	  status = 0;
-
-	  while ((status = mu_stream_read (istream, buf, bufsize, ioff, &nrd))
-		 == 0
-		 && nrd > 0)
+	  status = mu_stream_sequential_write (ostream, saved_envelope,
+					       strlen (saved_envelope));
+	  if (status == 0)
 	    {
-	      status = mu_stream_write (ostream, buf, nrd, off, &nwr);
-	      if (status)
-		break;
-	      ioff += nrd;
-	      off += nwr;
-	    }
+	      while ((status = mu_stream_sequential_read (istream,
+							  buf, bufsize,
+							  &nrd))
+		     == 0
+		     && nrd > 0)
+		{
+		  status = mu_stream_sequential_write (ostream, buf, nrd);
+		  if (status)
+		    break;
+		}
 	  
-	  free (buf);
+	      free (buf);
+	    }
 	}
       
       switch_user_id (auth, 0);
@@ -814,10 +934,12 @@ deliver (mu_mailbox_t imbx, char *name)
 	     original size */
 	  int rc = mu_stream_truncate (ostream, size);
 	  if (rc)
-	    mailer_err (_("Error writing to mailbox: %s. Mailbox NOT truncated: %s"),
+	    mailer_err ("write error", name,
+			_("Error writing to mailbox: %s. Mailbox NOT truncated: %s"),
 			mu_strerror (status), mu_strerror (rc));
 	  else  
-	    mailer_err (_("Error writing to mailbox: %s"),
+	    mailer_err ("write error", name,
+			_("Error writing to mailbox: %s"),
 	  	        mu_strerror (status));
 	}
     }
@@ -871,16 +993,22 @@ notify_biff (mu_mailbox_t mbox, char *name, size_t size)
 }
 
 void
-mailer_err (char *fmt, ...)
+mailer_err (const char *msg, const char *arg, const char *fmt, ...)
 {
   va_list ap;
 
   guess_retval (errno);
-  va_start (ap, fmt);
-  vfprintf (stderr, fmt, ap);
-  fprintf (stderr, "\n");
-  mu_verror (fmt, ap);
-  va_end (ap);
+  if (arg)
+    fprintf (stderr, "<%s>: %s", arg, msg);
+  else
+    fputs (msg, stderr);
+  fputc ('\n', stderr);
+  if (fmt)
+    {
+      va_start (ap, fmt);
+      mu_verror (fmt, ap);
+      va_end (ap);
+    }
 }
 
 int temp_errors[] = {
