@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
    Copyright (C) 1999, 2000, 2001, 2002, 2003, 
-   2005, 2006, 2007 Free Software Foundation, Inc.
+   2005, 2006, 2007, 2008 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -30,11 +30,11 @@
 #include <string.h>
 #include <unistd.h>
 #include <syslog.h>
+#include <sysexits.h>
 
 #include <mu_asprintf.h>
 #include <mailutils/argcv.h>
 #include <mailutils/libsieve.h>
-#include <mailutils/argp.h>
 #include <mailutils/auth.h>
 #include <mailutils/errno.h>
 #include <mailutils/folder.h>
@@ -46,6 +46,8 @@
 #include <mailutils/nls.h>
 #include <mailutils/tls.h>
 
+#include "mailutils/libargp.h"
+
 const char *program_version = "sieve (" PACKAGE_STRING ")";
 
 static char doc[] =
@@ -53,7 +55,7 @@ N_("GNU sieve -- a mail filtering tool")
 "\v"
 N_("Debug flags:\n\
   g - main parser traces\n\
-  T - mailutils traces (MU_DEBUG_TRACE)\n\
+  T - mailutils traces (MU_DEBUG_TRACE0-MU_DEBUG_TRACE1)\n\
   P - network protocols (MU_DEBUG_PROT)\n\
   t - sieve trace (MU_SIEVE_DEBUG_TRACE)\n\
   i - sieve instructions trace (MU_SIEVE_DEBUG_INSTR)\n");
@@ -61,6 +63,7 @@ N_("Debug flags:\n\
 #define D_DEFAULT "TPt"
 
 #define ARG_LINE_INFO 257
+#define ARG_NO_PROGRAM_NAME 258
 
 static struct argp_option options[] =
 {
@@ -80,7 +83,7 @@ static struct argp_option options[] =
    N_("Mailbox to sieve (defaults to user's mail spool)"), 0},
 
   {"ticket", 't', N_("TICKET"), 0,
-   N_("Ticket file for mailbox authentication"), 0},
+   N_("Ticket file for user authentication"), 0},
 
   {"debug", 'd', N_("FLAGS"), OPTION_ARG_OPTIONAL,
    N_("Debug flags (defaults to \"" D_DEFAULT "\")"), 0},
@@ -93,21 +96,22 @@ static struct argp_option options[] =
    
   {"email", 'e', N_("ADDRESS"), 0,
    N_("Override user email address"), 0},
+
+  {"no-program-name", ARG_NO_PROGRAM_NAME, NULL, 0,
+   N_("Do not prefix diagnostic messages with the program name"), 0},
   
   {0}
 };
 
-struct options {
-  int keep_going;
-  int compile_only;
-  char *mbox;
-  char *tickets;
-  int tickets_default;
-  int debug_level;
-  int sieve_debug;
-  int verbose;
-  char *script;
-};
+int keep_going;
+int compile_only;
+char *mbox_url;
+char *tickets;
+int tickets_default;
+int debug_level;
+int sieve_debug;
+int verbose;
+char *script;
 
 static int sieve_print_locus = 1; /* Should the log messages include the
 				     locus */
@@ -123,110 +127,124 @@ is_true_p (char *p)
   return rc;
 }
 
+static void
+set_debug_level (mu_debug_t debug, const char *arg)
+{
+  for (; *arg; arg++)
+    {
+      switch (*arg)
+	{
+	case 'T':
+	  debug_level |= MU_DEBUG_LEVEL_UPTO (MU_DEBUG_TRACE7);
+	  break;
+
+	case 'P':
+	  debug_level |= MU_DEBUG_LEVEL_MASK (MU_DEBUG_PROT);
+	  break;
+
+	case 'g':
+	  mu_sieve_yydebug = 1;
+	  break;
+
+	case 't':
+	  sieve_debug |= MU_SIEVE_DEBUG_TRACE;
+	  break;
+	  
+	case 'i':
+	  sieve_debug |= MU_SIEVE_DEBUG_INSTR;
+	  break;
+	  
+	default:
+	  if (debug)
+	    mu_cfg_format_error (debug, MU_DEBUG_ERROR,
+				 _("%c is not a valid debug flag"), *arg);
+	  else
+	    mu_error (_("%c is not a valid debug flag"), *arg);
+	}
+    }
+}
+
+int
+mu_compat_printer (void *data, mu_log_level_t level, const char *buf)
+{
+  fputs (buf, stderr);
+  return 0;
+}
+
 static error_t
 parser (int key, char *arg, struct argp_state *state)
 {
-  struct options *opts = state->input;
-  int rc;
+  static struct mu_argp_node_list lst;
   
   switch (key)
     {
-    case ARGP_KEY_INIT:
-      if (!opts->tickets)
-	{
-	  opts->tickets = mu_tilde_expansion ("~/.tickets", "/", NULL);
-	  opts->tickets_default = 1;
-	}
-      if (!opts->debug_level)
-	opts->debug_level = MU_DEBUG_ERROR;
-      log_facility = 0;
-      break;
-
     case 'e':
-      rc = mu_set_user_email (arg);
-      if (rc)
-	argp_error (state, _("Invalid email: %s"), mu_strerror (rc));
+      mu_argp_node_list_new (&lst, "email", arg);
       break;
       
     case 'n':
-      opts->sieve_debug |= MU_SIEVE_DRY_RUN;
+      sieve_debug |= MU_SIEVE_DRY_RUN;
       break;
 
     case 'k':
-      opts->keep_going = 1;
+      mu_argp_node_list_new (&lst, "keep-going", "yes");
       break;
 
     case 'c':
-      opts->compile_only = 1;
+      compile_only = 1;
       break;
 
     case 'D':
-      opts->compile_only = 2;
+      compile_only = 2;
       break;
       
     case 'f':
-      if (opts->mbox)
-	argp_error (state, _("Only one MBOX can be specified"));
-      opts->mbox = strdup (arg);
+      mu_argp_node_list_new (&lst, "mbox-url", arg);
       break;
       
     case 't':
-      free (opts->tickets);
-      opts->tickets = mu_tilde_expansion (arg, "/", NULL);
-      opts->tickets_default = 0;
+      mu_argp_node_list_new (&lst, "ticket", arg);
       break;
       
     case 'd':
-      if (!arg)
-	arg = D_DEFAULT;
-      for (; *arg; arg++)
-	{
-	  switch (*arg)
-	    {
-	    case 'T':
-	      opts->debug_level |= MU_DEBUG_TRACE;
-	      break;
-
-	    case 'P':
-	      opts->debug_level |= MU_DEBUG_PROT;
-	      break;
-
-	    case 'g':
-	      mu_sieve_yydebug = 1;
-	      break;
-
-	    case 't':
-	      opts->sieve_debug |= MU_SIEVE_DEBUG_TRACE;
-	      break;
-	      
-	    case 'i':
-	      opts->sieve_debug |= MU_SIEVE_DEBUG_INSTR;
-	      break;
-	      
-	    default:
-	      argp_error (state, _("%c is not a valid debug flag"), *arg);
-	      break;
-	    }
-	}
+      mu_argp_node_list_new (&lst, "debug", arg ? arg : D_DEFAULT);
       break;
 
     case 'v':
-      opts->verbose = 1;
+      mu_argp_node_list_new (&lst, "verbose", "yes");
       break;
 
     case ARG_LINE_INFO:
-      sieve_print_locus = is_true_p (arg);
+      mu_argp_node_list_new (&lst, "line-info",
+			     is_true_p (arg) ? "yes" : "no");
+      break;
+
+    case ARG_NO_PROGRAM_NAME:
+      {
+	mu_debug_t debug;
+	mu_diag_get_debug (&debug);
+	mu_debug_set_print (debug, mu_compat_printer, NULL);
+      }
       break;
       
     case ARGP_KEY_ARG:
-      if (opts->script)
+      if (script)
 	argp_error (state, _("Only one SCRIPT can be specified"));
-      opts->script = mu_tilde_expansion (arg, "/", NULL);
+      script = mu_tilde_expansion (arg, "/", NULL);
       break;
 
     case ARGP_KEY_NO_ARGS:
-      argp_error (state, _("SCRIPT must be specified"));
-
+      if (!mu_help_config_mode)
+	argp_error (state, _("SCRIPT must be specified"));
+      return ARGP_ERR_UNKNOWN;
+      
+    case ARGP_KEY_INIT:
+      mu_argp_node_list_init (&lst);
+      break;
+      
+    case ARGP_KEY_FINI:
+      mu_argp_node_list_finish (&lst, NULL, NULL);
+      break;
     default:
       return ARGP_ERR_UNKNOWN;
     }
@@ -242,221 +260,162 @@ static struct argp argp =
   doc
 };
 
+
+static int 
+cb_debug (mu_debug_t debug, void *data, mu_config_value_t *val)
+{
+  if (mu_cfg_assert_value_type (val, MU_CFG_STRING, debug))
+    return 1;
+  set_debug_level (debug, val->v.string);
+  return 0;
+}
+
+static int
+cb_email (mu_debug_t debug, void *data, mu_config_value_t *val)
+{
+  if (mu_cfg_assert_value_type (val, MU_CFG_STRING, debug))
+    return 1;
+  int rc = mu_set_user_email (val->v.string);
+  if (rc)
+    mu_cfg_format_error (debug, MU_DEBUG_ERROR, _("Invalid email: %s"),
+			 mu_strerror (rc));
+  return rc;
+}
+
+static int
+cb_ticket (mu_debug_t debug, void *data, mu_config_value_t *val)
+{
+  if (mu_cfg_assert_value_type (val, MU_CFG_STRING, debug))
+    return 1;
+  free (tickets);
+  tickets = mu_tilde_expansion (val->v.string, "/", NULL);
+  tickets_default = 0;
+  return 0;
+}
+
+static struct mu_cfg_param sieve_cfg_param[] = {
+  { "keep-going", mu_cfg_int, &keep_going, 0, NULL,
+    N_("Do not abort if execution fails on a message.") },
+  { "mbox-url", mu_cfg_string, &mbox_url, 0, NULL,
+    N_("Mailbox to sieve (defaults to user's mail spool)."),
+    N_("url") },
+  { "ticket", mu_cfg_callback, NULL, 0, cb_ticket,
+    N_("Ticket file for user authentication."),
+    N_("ticket") },
+  { "debug", mu_cfg_callback, NULL, 0, cb_debug,
+    N_("Debug flags.  Argument consists of one or more of the following "
+       "flags:\n"
+       "   g - main parser traces\n"
+       "   T - mailutils traces (MU_DEBUG_TRACE0-MU_DEBUG_TRACE1)\n"
+       "   P - network protocols (MU_DEBUG_PROT)\n"
+       "   t - sieve trace (MU_SIEVE_DEBUG_TRACE)\n"
+       "   i - sieve instructions trace (MU_SIEVE_DEBUG_INSTR).") },
+  { "verbose", mu_cfg_bool, &verbose, 0, NULL,
+    N_("Log all executed actions.") },
+  { "line-info", mu_cfg_bool, &sieve_print_locus, 0, NULL,
+    N_("Print source locations along with action logs (default).") },
+  { "email", mu_cfg_callback, NULL, 0, cb_email,
+    N_("Set user email address.") },
+  { NULL }
+};
+
+
 static const char *sieve_argp_capa[] =
 {
   "common",
+  "debug",
   "mailbox",
+  "locking",
   "license",
   "logging",
   "mailer",
-  "sieve",
-#ifdef WITH_TLS
-  "tls",
-#endif
   NULL
 };
 
 static int
-sieve_stderr_debug_printer (void *unused, const char *fmt, va_list ap)
+_sieve_debug_printer (void *unused, const char *fmt, va_list ap)
 {
-  vfprintf (stderr, fmt, ap);
-  return 0;
-}
-
-static int
-sieve_syslog_debug_printer (void *unused, const char *fmt, va_list ap)
-{
-  vsyslog (LOG_DEBUG, fmt, ap);
-  return 0;
-}
-
-static int
-stderr_debug_print (mu_debug_t unused, size_t level, const char *fmt,
-		    va_list ap)
-{
-  vfprintf ((level == MU_DEBUG_ERROR) ? stderr : stdout, fmt, ap);
-  return 0;
-}
-
-static int
-syslog_debug_print (mu_debug_t unused, size_t level, const char *fmt,
-		    va_list ap)
-{
-  vsyslog ((level == MU_DEBUG_ERROR) ? LOG_ERR : LOG_DEBUG, fmt, ap);
+  mu_diag_vprintf (MU_DIAG_DEBUG, fmt, ap);
   return 0;
 }
 
 static void
-stdout_action_log (void *unused,
-		   const mu_sieve_locus_t *locus, size_t msgno, mu_message_t msg,
+_sieve_action_log (void *unused,
+		   const mu_sieve_locus_t *locus, size_t msgno,
+		   mu_message_t msg,
 		   const char *action, const char *fmt, va_list ap)
 {
   size_t uid = 0;
-  
-  mu_message_get_uid (msg, &uid);
+  mu_debug_t debug;
 
   if (sieve_print_locus)
-    fprintf (stdout, _("%s:%lu: %s on msg uid %lu"),
-	     locus->source_file, (unsigned long) locus->source_line,
-	     action, (unsigned long) uid);
-  else
-    fprintf (stdout, _("%s on msg uid %lu"), action, (unsigned long) uid);
+    {
+      mu_diag_get_debug (&debug);
+      mu_debug_set_locus (debug, locus->source_file, locus->source_line);
+    }
+  
+  mu_message_get_uid (msg, &uid);
+  mu_diag_printf (MU_DIAG_NOTICE, _("%s on msg uid %d"), action, uid);
   
   if (fmt && strlen (fmt))
     {
-      fprintf (stdout, ": ");
-      vfprintf (stdout, fmt, ap);
+      mu_diag_printf (MU_DIAG_NOTICE, ": ");
+      mu_diag_vprintf (MU_DIAG_NOTICE, fmt, ap);
     }
-  fprintf (stdout, "\n");
-}
-
-static void
-syslog_action_log (void *unused,
-		   const mu_sieve_locus_t *locus, size_t msgno, mu_message_t msg,
-		   const char *action, const char *fmt, va_list ap)
-{
-  size_t uid = 0;
-  char *text = NULL;
-  
-  mu_message_get_uid (msg, &uid);
-
+  mu_diag_printf (MU_DIAG_NOTICE, "\n");
   if (sieve_print_locus)
-    asprintf (&text, _("%s:%lu: %s on msg uid %lu"),
-	      locus->source_file, (unsigned long) locus->source_line,
-	      action, (unsigned long) uid);
-  else
-    asprintf (&text, _("%s on msg uid %lu"), action, (unsigned long) uid);
-    
-  if (fmt && strlen (fmt))
-    {
-      char *diag = NULL;
-      asprintf (&diag, fmt, ap);
-      syslog (LOG_NOTICE, "%s: %s", text, diag);
-      free (diag);
-    }
-  else
-    syslog (LOG_NOTICE, "%s", text);
-  free (text);
+    mu_debug_set_locus (debug, NULL, 0);
 }
 
-int
-main (int argc, char *argv[])
+static int
+sieve_message (mu_sieve_machine_t mach)
 {
-  mu_sieve_machine_t mach;
-  mu_wicket_t wicket = 0;
-  mu_ticket_t ticket = 0;
-  mu_debug_t debug = 0;
-  mu_mailbox_t mbox = 0;
   int rc;
-  struct options opts = {0};
-  int (*debugfp) (mu_debug_t, size_t level, const char *, va_list);
+  mu_stream_t instr;
+  mu_message_t msg;
+  mu_attribute_t attr;
 
-  /* Native Language Support */
-  mu_init_nls ();
-
-  mu_argp_init (program_version, NULL);
-#ifdef WITH_TLS
-  mu_tls_init_client_argp ();
-#endif
-  mu_sieve_argp_init ();
-  mu_register_all_formats ();
-  
-  rc = mu_argp_parse (&argp, &argc, &argv, ARGP_IN_ORDER, sieve_argp_capa,
-		      0, &opts);
-
-  if (rc)
-    return 1;
-
-  mu_register_all_formats ();
-
-  /* Sieve interpreter setup. */
-  rc = mu_sieve_machine_init (&mach, NULL);
+  rc = mu_stdio_stream_create (&instr, stdin, MU_STREAM_SEEKABLE);
   if (rc)
     {
-      mu_error (_("Cannot initialize sieve machine: %s"), mu_strerror (rc));
-      return 1;
+      mu_error (_("Cannot create stream: %s"), mu_strerror (rc));
+      return EX_SOFTWARE;
     }
-
-  if (log_facility)
-    {
-      openlog ("sieve", LOG_PID, log_facility);
-      mu_error_set_print (mu_syslog_error_printer);
-      mu_sieve_set_debug (mach, sieve_syslog_debug_printer);
-      if (opts.verbose)
-	mu_sieve_set_logger (mach, syslog_action_log);
-      debugfp = syslog_debug_print;
-    }
-  else
-    {
-      mu_sieve_set_debug (mach, sieve_stderr_debug_printer);
-      if (opts.verbose)
-	mu_sieve_set_logger (mach, stdout_action_log);
-      debugfp = stderr_debug_print;
-    }
-  
-  rc = mu_sieve_compile (mach, opts.script);
+  rc = mu_stream_open (instr);
   if (rc)
-    return 1;
-
-  /* We can finish if its only a compilation check. */
-  if (opts.compile_only)
     {
-      if (opts.compile_only == 2)
-	mu_sieve_disass (mach);
-      return 0;
+      mu_error (_("Cannot open stream: %s"), mu_strerror (rc));
+      return EX_SOFTWARE;
     }
-
-  /* Create a ticket, if we can. */
-  if (opts.tickets)
+  rc = mu_stream_to_message (instr, &msg);
+  if (rc)
     {
-      if ((rc = mu_wicket_create (&wicket, opts.tickets)) == 0)
-        {
-          if ((rc = mu_wicket_get_ticket (wicket, &ticket, 0, 0)) != 0)
-            {
-              mu_error (_("ticket_get failed: %s"), mu_strerror (rc));
-              goto cleanup;
-            }
-        }
-      else if (!(opts.tickets_default && errno == ENOENT))
-        {
-          mu_error (_("mu_wicket_create `%s' failed: %s"),
-                    opts.tickets, mu_strerror (rc));
-          goto cleanup;
-        }
-      if (ticket)
-        mu_sieve_set_ticket (mach, ticket);
+      mu_error (_("Cannot create message from stream: %s"),
+		mu_strerror (rc));
+      return EX_SOFTWARE;
     }
+  mu_message_get_attribute (msg, &attr);
+  mu_attribute_unset_deleted (attr);
+  rc = mu_sieve_message (mach, msg);
+  if (rc)
+    /* FIXME: switch (rc)...*/
+    return EX_SOFTWARE;
 
-  /* Create a debug object, if needed. */
-  if (opts.debug_level)
-    {
-      if ((rc = mu_debug_create (&debug, mach)))
-	{
-	  mu_error (_("mu_debug_create failed: %s"), mu_strerror (rc));
-	  goto cleanup;
-	}
-      if ((rc = mu_debug_set_level (debug, opts.debug_level)))
-	{
-	  mu_error (_("mu_debug_set_level failed: %s"),
-		    mu_strerror (rc));
-	  goto cleanup;
-	}
-      if ((rc = mu_debug_set_print (debug, debugfp, mach)))
-	{
-	  mu_error (_("mu_debug_set_print failed: %s"),
-		    mu_strerror (rc));
-	  goto cleanup;
-	}
-    }
+  return mu_attribute_is_deleted (attr) ? 1 : EX_OK;
+}
+
+static int
+sieve_mailbox (mu_sieve_machine_t mach, mu_ticket_t ticket, mu_debug_t debug)
+{
+  int rc;
+  mu_mailbox_t mbox = NULL;
   
-  mu_sieve_set_debug_level (mach, debug, opts.sieve_debug);
-    
   /* Create, give a ticket to, and open the mailbox. */
-  if ((rc = mu_mailbox_create_default (&mbox, opts.mbox)) != 0)
+  if ((rc = mu_mailbox_create_default (&mbox, mbox_url)) != 0)
     {
-      if (opts.mbox)
+      if (mbox)
 	mu_error (_("Could not create mailbox `%s': %s"),
-		  opts.mbox, mu_strerror (rc));
+		  mbox_url, mu_strerror (rc));
       else
 	mu_error (_("Could not create default mailbox: %s"),
 		  mu_strerror (rc));
@@ -473,65 +432,70 @@ main (int argc, char *argv[])
     {
       mu_folder_t folder = NULL;
       mu_authority_t auth = NULL;
-
+      
       if ((rc = mu_mailbox_get_folder (mbox, &folder)))
 	{
 	  mu_error (_("mu_mailbox_get_folder failed: %s"),
-		   mu_strerror (rc));
+		    mu_strerror (rc));
 	  goto cleanup;
 	}
-
+      
       if ((rc = mu_folder_get_authority (folder, &auth)))
 	{
 	  mu_error (_("mu_folder_get_authority failed: %s"),
-		   mu_strerror (rc));
+		    mu_strerror (rc));
 	  goto cleanup;
 	}
-
+      
       /* Authentication-less folders don't have authorities. */
       if (auth && (rc = mu_authority_set_ticket (auth, ticket)))
 	{
 	  mu_error (_("mu_authority_set_ticket failed: %s"),
-		   mu_strerror (rc));
+		    mu_strerror (rc));
 	  goto cleanup;
 	}
     }
-
+  
   /* Open the mailbox read-only if we aren't going to modify it. */
-  if (opts.sieve_debug & MU_SIEVE_DRY_RUN)
+  if (sieve_debug & MU_SIEVE_DRY_RUN)
     rc = mu_mailbox_open (mbox, MU_STREAM_READ);
   else
     rc = mu_mailbox_open (mbox, MU_STREAM_RDWR);
-
+  
   if (rc != 0)
     {
-      if (opts.mbox)
-	mu_error (_("Opening mailbox `%s' failed: %s"),
-		  opts.mbox, mu_strerror (rc));
+      if (mbox)
+	{
+	  mu_url_t url = NULL;
+
+	  mu_mailbox_get_url (mbox, &url);
+	  mu_error (_("Opening mailbox `%s' failed: %s"),
+		    mu_url_to_string (url), mu_strerror (rc));
+	}
       else
 	mu_error (_("Opening default mailbox failed: %s"),
 		  mu_strerror (rc));
       mu_mailbox_destroy (&mbox);
       goto cleanup;
     }
-
+  
   /* Process the mailbox */
   rc = mu_sieve_mailbox (mach, mbox);
-
-cleanup:
-  if (mbox && !(opts.sieve_debug & MU_SIEVE_DRY_RUN))
+  
+ cleanup:
+  if (mbox && !(sieve_debug & MU_SIEVE_DRY_RUN))
     {
       int e;
-
+      
       /* A message won't be marked deleted unless the script executed
          succesfully on it, so we always do an expunge, it will delete
          any messages that were marked DELETED even if execution failed
          on a later message. */
       if ((e = mu_mailbox_expunge (mbox)) != 0)
 	{
-	  if (opts.mbox)
+	  if (mbox)
 	    mu_error (_("Expunge on mailbox `%s' failed: %s"),
-		      opts.mbox, mu_strerror (e));
+		      mbox_url, mu_strerror (e));
 	  else
 	    mu_error (_("Expunge on default mailbox failed: %s"),
 		      mu_strerror (e));
@@ -540,12 +504,122 @@ cleanup:
       if (e && !rc)
 	rc = e;
     }
-
+  
   mu_sieve_machine_destroy (&mach);
   mu_mailbox_close (mbox);
   mu_mailbox_destroy (&mbox);
+  /* FIXME: switch (rc) ... */
+  return rc ? EX_SOFTWARE : EX_OK;
+}
+
+int
+main (int argc, char *argv[])
+{
+  mu_sieve_machine_t mach;
+  mu_wicket_t wicket = 0;
+  mu_ticket_t ticket = 0;
+  mu_debug_t debug = 0;
+  int rc;
+
+  /* Native Language Support */
+  MU_APP_INIT_NLS ();
+
+  mu_argp_init (program_version, NULL);
+#ifdef WITH_TLS
+  mu_gocs_register ("tls", mu_tls_module_init);
+#endif
+  mu_gocs_register ("sieve", mu_sieve_module_init);
+
+  mu_register_all_formats ();
+
+  tickets = mu_tilde_expansion ("~/.tickets", "/", NULL);
+  tickets_default = 1;
+  debug_level = MU_DEBUG_LEVEL_MASK (MU_DEBUG_ERROR);
+  mu_log_facility = 0;
+
+  if (mu_app_init (&argp, sieve_argp_capa, sieve_cfg_param, 
+		   argc, argv, ARGP_IN_ORDER, NULL, NULL))
+    exit (EX_USAGE);
+
+  /* Sieve interpreter setup. */
+  rc = mu_sieve_machine_init (&mach, NULL);
+  if (rc)
+    {
+      mu_error (_("Cannot initialize sieve machine: %s"), mu_strerror (rc));
+      return EX_SOFTWARE;
+    }
+
+  if (mu_log_facility)
+    {
+      mu_debug_t debug;
+
+      mu_diag_get_debug (&debug);
+      openlog (MU_LOG_TAG (), LOG_PID, mu_log_facility);
+      mu_debug_set_print (debug, mu_diag_syslog_printer, NULL);
+    }
+
+  mu_sieve_set_debug (mach, _sieve_debug_printer);
+  if (verbose)
+    mu_sieve_set_logger (mach, _sieve_action_log);
+  
+  rc = mu_sieve_compile (mach, script);
+  if (rc)
+    return EX_CONFIG;
+
+  /* We can finish if its only a compilation check. */
+  if (compile_only)
+    {
+      if (compile_only == 2)
+	mu_sieve_disass (mach);
+      return EX_OK;
+    }
+
+  /* Create a ticket, if we can. */
+  if (tickets)
+    {
+      if ((rc = mu_wicket_create (&wicket, tickets)) == 0)
+        {
+          if ((rc = mu_wicket_get_ticket (wicket, &ticket, 0, 0)) != 0)
+            {
+              mu_error (_("ticket_get failed: %s"), mu_strerror (rc));
+              return EX_SOFTWARE; /* FIXME: really? */
+            }
+        }
+      else if (!(tickets_default && errno == ENOENT))
+        {
+          mu_error (_("mu_wicket_create `%s' failed: %s"),
+                    tickets, mu_strerror (rc));
+          return EX_SOFTWARE;
+        }
+      if (ticket)
+        mu_sieve_set_ticket (mach, ticket);
+    }
+
+  /* Create a debug object, if needed. */
+  if (debug_level)
+    {
+      if ((rc = mu_debug_create (&debug, mach)))
+	{
+	  mu_error (_("mu_debug_create failed: %s"), mu_strerror (rc));
+	  return EX_SOFTWARE;
+	}
+      if ((rc = mu_debug_set_level (debug, debug_level)))
+	{
+	  mu_error (_("mu_debug_set_level failed: %s"),
+		    mu_strerror (rc));
+	  return EX_SOFTWARE;
+	}
+      mu_sieve_set_debug_object (mach, debug);
+    }
+  
+  mu_sieve_set_debug_level (mach, sieve_debug);
+
+  if (mbox_url && strcmp (mbox_url, "-") == 0)
+    rc = sieve_message (mach);
+  else
+    rc = sieve_mailbox (mach, ticket, debug);
   mu_debug_destroy (&debug, mach);
 
-  return rc ? 1 : 0;
+  return rc;
 }
 
